"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const minimist_1 = __importDefault(require("minimist"));
const minimist_options_1 = __importDefault(require("minimist-options"));
const nodom_1 = require("nodom");
const process_1 = require("process");
const ipynb2html = __importStar(require("ipynb2html"));
const template_1 = __importDefault(require("./template"));
const { version, bugs: bugsUrl } = { "version": "0.1.0-beta.8", "bugs": "https://github.com/jirutka/ipynb2html/issues" };
const progName = 'ipynb2html';
const helpMsg = `\
Usage: ${progName} [options] <input> [<output>]

Convert Jupyter Notebook 4.0+ to a static HTML page.

Arguments:
  <input>          Path of the Jupyter notebook to read, or "-" for STDIN.
  <output>         Path of the file to write the output HTML into. If not
                   provided, the output will be written to STDOUT.

Options:
  -d --debug       Print debug messages.
  -h --help        Show this message and exit.
  -V --version     Print version and exit.

Exit Codes:
  1                Generic error code.
  2                Missing required arguments or invalid option.

Please report bugs at <${bugsUrl}>.
`;
function logErr(msg) {
    console.error(`${progName}: ${msg}`);
}
function parseCliArgs(argv) {
    const opts = minimist_1.default(argv, minimist_options_1.default({
        debug: { alias: 'd', type: 'boolean' },
        version: { alias: 'V', type: 'boolean' },
        help: { alias: 'h', type: 'boolean' },
        arguments: 'string',
        stopEarly: true,
        unknown: (arg) => {
            if (arg.startsWith('-')) {
                logErr(`Unknown option: ${arg}`);
                return process_1.exit(2);
            }
            else {
                return true;
            }
        },
    }));
    if (opts.help) {
        console.log(helpMsg);
        return process_1.exit(0);
    }
    if (opts.version) {
        console.log(`${progName} ${version}`);
        return process_1.exit(0);
    }
    if (opts._.length < 1 || opts._.length > 2) {
        logErr('Invalid number of arguments\n');
        console.log(helpMsg);
        return process_1.exit(2);
    }
    const [input, output] = opts._;
    return {
        debug: opts.debug,
        input: input === '-' ? 0 : input,
        output,
    };
}
const cli = (argv) => {
    const opts = parseCliArgs(argv);
    try {
        const notebook = JSON.parse(fs_1.default.readFileSync(opts.input, 'utf-8'));
        const title = ipynb2html.readNotebookTitle(notebook) || 'Notebook';
        const render = ipynb2html.createRenderer(new nodom_1.Document());
        const contents = render(notebook).outerHTML;
        const html = template_1.default(contents, title);
        if (opts.output) {
            fs_1.default.writeFileSync(opts.output, html);
        }
        else {
            console.log(html);
        }
    }
    catch (err) {
        if (opts.debug) {
            console.debug(err);
        }
        else {
            logErr(err.message);
        }
        return process_1.exit(1);
    }
};
exports.default = cli;
//# sourceMappingURL=cli.js.map